use strict;

use Test::More;
ok(1, "More tests soon");

done_testing();

__END__

=head1 NAME

0.9-generate.t - tests for generating RSS 0.90 data with XML::RSS::LibXML.pm

=head1 SYNOPSIS

 use Test::Harness qw (runtests);
 runtests (./XML-RSS/t/*.t);

=head1 DESCRIPTION

Tests for generating RSS 0.90 data with XML::RSS::LibXML.pm

=head1 VERSION

$Revision: 1.2 $

=head1 DATE

$Date: 2003/02/20 17:12:44 $

=head1 AUTHOR

Aaron Straup Cope

=head1 SEE ALSO

http://www.purplepages.ie/RSS/netscape/rss0.90.html

=cut
